#ifndef SOCKET_H_
#define SOCKET_H_
#include <sys/socket.h>
/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Public Functions
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Accept a connection on a socket.
 * @param sockfd        Socket descriptor.
 * @param addr          IP address.
 * @param addrlen       Length of IP address.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return returns a non-negative integer that is a descriptor for the accepted socket. On error, -1 is returned.
 */
int Accept ( int sockfd, struct sockaddr* addr, socklen_t* addrlen, const char* filename, int line );


/**
 * Bind a name to a socket.
 * @param sockfd        Socket descriptor.
 * @param addr          IP address.
 * @param addrlen       Length of IP address.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Bind ( int sockfd, const struct sockaddr* my_addr, socklen_t addrlen, const char* filename, int line );


/**
 * Close a file descriptor.
 * @param fd            File descriptor.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Close ( int fd, const char* filename, int line );


/**
 * Initiate a connection on a socket.
 * @param sockfd        Socket descriptor.
 * @param addr          IP address of server.
 * @param addrlen       Length of IP address.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Connect ( int sockfd, const struct sockaddr* serv_addr, socklen_t addrlen, const char* filename, int line );


/**
 * Get the 4th position of IPv4 in A.B.C.D format.
 * @return Returns an unsigned integer from 0-255 on success; otherwise, returns -1.
 */
int IP4th ();


/**
 * Listen for connections on a socket.
 * @param sockfd        Socket descriptor.
 * @param backlog       Define the maximum length the queue of pending connections may grow to.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Listen ( int sockfd, int backlog, const char* filename, int line );


/**
 * Read number of bytes on a descriptor.
 * @param fd            Descriptor.
 * @param ptr           Pointer to receive buffer.
 * @param nbytes        Maximum number of bytes can be read.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns number of bytes read on success. Otherwise, returns -1.
 */
ssize_t Read ( int fd, void *ptr, size_t nbytes, const char* filename, int line );


/**
 * Send a message on a socket.
 * @param socket        Specifies the socket file descriptor.
 * @param message       Points to a buffer containing the message to be sent.
 * @param length        Specifies the size of the message in bytes.
 * @param flags         Specifies the type of message transmission.
 * @param dest_addr     Points to a sockaddr structure containing the destination address.
 * @param dest_len      Specifies the length of the sockaddr structure pointed to by the @dest_addr argument.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns the number of bytes sent. Otherwise, -1 shall be returned if any error.
 */
//ssize_t Sendto ( int socket, const void* message, size_t length, int flags, const struct sockaddr* dest_addr, socklen_t dest_len,
//    const char* filename, int line );


/**
 * synchronous I/O multiplexing.
 * @param nfds          The highest-numbered file descriptor.
 * @param readfds       The @readfds will be watched to see if characters become available for reading.
 * @param writefds      It will be watched to see if a write will not block.
 * @param exceptfds     It will be watched for exceptions.
 * @param timeout       It is an upper bound on the amount of time elapsed before select() returns.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return On success, returns the number of file descriptors contained in the three returned descriptor sets
 *         (that is, the total number of bits that are set in readfds, writefds, exceptfds)
 *         which may be zero if the timeout expires before anything interesting happens.
 *         On error, -1 is returned.
 */
int Select ( int nfds, fd_set* readfds, fd_set* writefds, fd_set* exceptfds, struct timeval* timeout, const char* filename, int line );


/**
 * Set options on a socket.
 * @param s             Socket descriptor.
 * @param level         Socket level.
 * @param optname       Uninterpreted to the appropriate protocol module for interpretation.
 * @param optval        Initially containing the size of the buffer pointed to by @optval,
 *                      and modified on return to indicate the actual size of the value returned.
 *                      If no option value is to be supplied or returned, @optval may be NULL.
 * @param optlen        Size of @optval.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns 0 on success; returns -1 on failure.
 */
int Setsockopt ( int s, int level, int optname, const void* optval, socklen_t optlen, const char* filename, int line );


/**
 * Create an end-point for communication.
 * @param domain        Communication domain.
 * @param type          Communication semantics.
 * @param protocol      Protocol to be used with the socket.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns a socket descriptor.
 */
int Socket ( int domain, int type, int protocol, const char* filename, int line );


/**
 * @param socket        Specifies the socket file descriptor.
 * @param buffer        Points to the buffer where the message should be stored.
 * @param length        Specifies the length in bytes of the buffer pointed to by the @buffer argument.
 * @param flags         Specifies the type of message reception.
 * @param address       A null pointer, or points to a sockaddr structure in which the sending address is to be stored.
 * @param address_len   Specifies the length of the sockaddr structure pointed to by the address argument.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 * @return Returns length of the message in bytes; If no messages are available to be received and the peer has performed an orderly shutdown,
 *         then returns 0. Otherwise, returns -1 if any error.
 */
//ssize_t Recvfrom ( int socket, void* buffer, size_t length, int flags, struct sockaddr* address, socklen_t* address_len,
//    const char* filename, int line );


/**
 * Write number of bytes to a descriptor.
 * @param fd            Descriptor.
 * @param nbytes        Number of bytes to be written.
 * @param filename      File name from source code.
 * @param line          Line number from source code.
 */
void Writen ( int fd, void *ptr, size_t nbytes, const char* filename, int line );




#endif /* SOCKET_H_ */
